# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Brand"]


class Brand(BaseModel):
    name: Optional[str] = None
    """The name of the brand."""

    description: Optional[str] = None
    """Description of the brand."""

    logo: Optional[str] = None
    """URL of the brand's logo."""

    same_as: Optional[List[str]] = FieldInfo(alias="sameAs", default=None)
    """URLs to external references for the brand."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)

    url: Optional[str] = None
    """The brand's official website."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CatalogStyleAndTagsSearchParams"]


class CatalogStyleAndTagsSearchParams(TypedDict, total=False):
    type: Required[str]

    styles: Required[List[str]]

    tags: Required[List[str]]

    compact_mode: Optional[Literal["compact", "medium"]]

    limit: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ThreeDModel", "Encoding"]


class Encoding(BaseModel):
    content_url: Optional[str] = FieldInfo(alias="contentUrl", default=None)
    """URL to the actual content of the media object."""

    context: Optional[str] = None

    encoding_format: Optional[str] = FieldInfo(alias="encodingFormat", default=None)
    """The media format or mime type."""

    name: Optional[str] = None
    """The name of the media object."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)

    upload_date: Optional[datetime] = FieldInfo(alias="uploadDate", default=None)
    """The date the media was uploaded."""


class ThreeDModel(BaseModel):
    author: Optional[str] = None
    """Author of the work."""

    content_size: Optional[str] = FieldInfo(alias="contentSize", default=None)
    """File size in megabytes or gigabytes."""

    content_url: Optional[str] = FieldInfo(alias="contentUrl", default=None)
    """URL to the actual content of the media object."""

    context: Optional[str] = None

    creator: Optional[str] = None
    """The creator of the 3D model."""

    date_published: Optional[datetime] = FieldInfo(alias="datePublished", default=None)
    """Date of first publication."""

    embed_url: Optional[str] = FieldInfo(alias="embedUrl", default=None)
    """A URL pointing to a player for the 3D model."""

    encoding: Optional[List[Encoding]] = None
    """A media object representing the 3D model."""

    encoding_format: Optional[str] = FieldInfo(alias="encodingFormat", default=None)
    """The file format of the 3D model (e.g., 'model/gltf+json')."""

    interaction_count: Optional[int] = FieldInfo(alias="interactionCount", default=None)
    """Number of interactions for the 3D model."""

    is_based_on_url: Optional[str] = FieldInfo(alias="isBasedOnUrl", default=None)
    """A related resource that the 3D model is based on."""

    license: Optional[str] = None
    """License under which the work is published."""

    material: Optional[str] = None
    """The material used to create the 3D model."""

    name: Optional[str] = None
    """The name of the media object."""

    thumbnail_url: Optional[List[str]] = FieldInfo(alias="thumbnailUrl", default=None)
    """A URL pointing to the model thumbnail image."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)

    upload_date: Optional[datetime] = FieldInfo(alias="uploadDate", default=None)
    """The date the media was uploaded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CatalogTextSearchParams", "Facet"]


class CatalogTextSearchParams(TypedDict, total=False):
    text: Required[str]
    """
    The text is converted to a vector embedding and used to search for products in
    the e-commerce catalog with pre-computed product embeddings.
    """

    facets: Optional[Iterable[Facet]]
    """The search results will be filtered by the specified facets."""

    limit: int
    """The maximum number of results to return from the search. The default is 10."""

    price_max: Optional[float]
    """
    The products will be filtered to have a price less than or equal to the
    specified value.
    """

    price_min: Optional[float]
    """
    The products will be filtered to have a price greater than or equal to the
    specified value.
    """


class Facet(TypedDict, total=False):
    name: Required[Literal["brand_name", "product_type"]]

    values: Required[List[str]]
    """List of values to filter by.

    They should all be lowercase. Facet values can be phrases, so make sure to
    include the spaces.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .context_enum import ContextEnum

__all__ = ["BreadcrumbList", "ItemListElement", "ItemListElementItem"]


class ItemListElementItem(BaseModel):
    id: str = FieldInfo(alias="id_")

    name: str


class ItemListElement(BaseModel):
    item: ItemListElementItem

    position: int

    type: str = FieldInfo(alias="type_")


class BreadcrumbList(BaseModel):
    context: Optional[ContextEnum] = None

    item_list_element: Optional[List[ItemListElement]] = FieldInfo(alias="itemListElement", default=None)
    """The list of breadcrumb items."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .catalog import (
    CatalogResource,
    AsyncCatalogResource,
    CatalogResourceWithRawResponse,
    AsyncCatalogResourceWithRawResponse,
    CatalogResourceWithStreamingResponse,
    AsyncCatalogResourceWithStreamingResponse,
)

__all__ = [
    "CatalogResource",
    "AsyncCatalogResource",
    "CatalogResourceWithRawResponse",
    "AsyncCatalogResourceWithRawResponse",
    "CatalogResourceWithStreamingResponse",
    "AsyncCatalogResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .organization import Organization
from .quantitative_value import QuantitativeValue

__all__ = [
    "Offer",
    "PriceSpecification",
    "PriceSpecificationPriceSpecification",
    "PriceSpecificationCompoundPriceSpecification",
    "PriceSpecificationCompoundPriceSpecificationPriceComponent",
]


class PriceSpecificationPriceSpecification(BaseModel):
    price: Optional[float] = None
    """Price of the offer."""

    price_currency: Optional[str] = FieldInfo(alias="priceCurrency", default=None)
    """Currency of the price."""

    valid_from: Optional[datetime] = FieldInfo(alias="validFrom", default=None)
    """The start of the validity period for the price."""

    valid_through: Optional[datetime] = FieldInfo(alias="validThrough", default=None)
    """The end of the validity period for the price."""


class PriceSpecificationCompoundPriceSpecificationPriceComponent(BaseModel):
    price: Optional[float] = None
    """Price of the offer."""

    price_currency: Optional[str] = FieldInfo(alias="priceCurrency", default=None)
    """Currency of the price."""

    price_type: Optional[str] = FieldInfo(alias="priceType", default=None)
    """
    The type of price specification as enumerated in schema.org's
    PriceTypeEnumeration, for example http://schema.org/ListPrice,
    http://schema.org/RegularPrice, http://schema.org/SalePrice, etc.
    """

    valid_from: Optional[datetime] = FieldInfo(alias="validFrom", default=None)
    """The start of the validity period for the price."""

    valid_through: Optional[datetime] = FieldInfo(alias="validThrough", default=None)
    """The end of the validity period for the price."""


class PriceSpecificationCompoundPriceSpecification(BaseModel):
    price_component: Optional[List[PriceSpecificationCompoundPriceSpecificationPriceComponent]] = FieldInfo(
        alias="priceComponent", default=None
    )
    """A list of unit price specifications for the item or offer."""


PriceSpecification: TypeAlias = Union[
    PriceSpecificationPriceSpecification, PriceSpecificationCompoundPriceSpecification, None
]


class Offer(BaseModel):
    availability: Optional[str] = None
    """
    The availability of the product — for example http://schema.org/InStock,
    http://schema.org/OutOfStock, http://schema.org/PreOrder, etc.
    """

    availability_ends: Optional[datetime] = FieldInfo(alias="availabilityEnds", default=None)
    """End time of availability."""

    availability_starts: Optional[datetime] = FieldInfo(alias="availabilityStarts", default=None)
    """Start time of availability."""

    eligible_quantity: Optional[QuantitativeValue] = FieldInfo(alias="eligibleQuantity", default=None)
    """Schema.org QuantitativeValue model."""

    inventory_level: Optional[QuantitativeValue] = FieldInfo(alias="inventoryLevel", default=None)
    """Schema.org QuantitativeValue model."""

    item_condition: Optional[str] = FieldInfo(alias="itemCondition", default=None)
    """Condition of the item."""

    name: Optional[str] = None
    """Name of the offer."""

    price_specification: Optional[PriceSpecification] = FieldInfo(alias="priceSpecification", default=None)
    """Price specification for the product."""

    seller: Optional[Organization] = None
    """Schema.org model for Organization."""

    sku: Optional[str] = None
    """SKU of the product."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["Rating"]


class Rating(BaseModel):
    average_rating: Optional[float] = None
    """Average rating, scaled between 1-5."""

    rating_count: Optional[int] = None
    """Total number of ratings. Must be nonnegative."""

    rating_histogram: Optional[List[int]] = None
    """List of rating counts per rating value."""

    type: Optional[Literal["AggregateRating", "Rating"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from octogen.api import OctogenAPI, AsyncOctogenAPI
from tests.utils import assert_matches_type
from octogen.api.types import (
    SearchToolOutput,
    CatalogUploadFileResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCatalog:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_agent_search(self, client: OctogenAPI) -> None:
        catalog = client.catalog.agent_search(
            text="text",
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_agent_search_with_all_params(self, client: OctogenAPI) -> None:
        catalog = client.catalog.agent_search(
            text="text",
            limit=0,
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_agent_search(self, client: OctogenAPI) -> None:
        response = client.catalog.with_raw_response.agent_search(
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = response.parse()
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_agent_search(self, client: OctogenAPI) -> None:
        with client.catalog.with_streaming_response.agent_search(
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = response.parse()
            assert_matches_type(SearchToolOutput, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_file(self, client: OctogenAPI) -> None:
        catalog = client.catalog.retrieve_file(
            "file_id",
        )
        assert_matches_type(object, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_file(self, client: OctogenAPI) -> None:
        response = client.catalog.with_raw_response.retrieve_file(
            "file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = response.parse()
        assert_matches_type(object, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_file(self, client: OctogenAPI) -> None:
        with client.catalog.with_streaming_response.retrieve_file(
            "file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = response.parse()
            assert_matches_type(object, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve_file(self, client: OctogenAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            client.catalog.with_raw_response.retrieve_file(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_style_and_tags_search(self, client: OctogenAPI) -> None:
        catalog = client.catalog.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_style_and_tags_search_with_all_params(self, client: OctogenAPI) -> None:
        catalog = client.catalog.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
            compact_mode="compact",
            limit=0,
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_style_and_tags_search(self, client: OctogenAPI) -> None:
        response = client.catalog.with_raw_response.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = response.parse()
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_style_and_tags_search(self, client: OctogenAPI) -> None:
        with client.catalog.with_streaming_response.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = response.parse()
            assert_matches_type(SearchToolOutput, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_text_search(self, client: OctogenAPI) -> None:
        catalog = client.catalog.text_search(
            text="text",
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_text_search_with_all_params(self, client: OctogenAPI) -> None:
        catalog = client.catalog.text_search(
            text="text",
            facets=[
                {
                    "name": "brand_name",
                    "values": ["string"],
                }
            ],
            limit=0,
            price_max=0,
            price_min=0,
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_text_search(self, client: OctogenAPI) -> None:
        response = client.catalog.with_raw_response.text_search(
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = response.parse()
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_text_search(self, client: OctogenAPI) -> None:
        with client.catalog.with_streaming_response.text_search(
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = response.parse()
            assert_matches_type(SearchToolOutput, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_upload_file(self, client: OctogenAPI) -> None:
        catalog = client.catalog.upload_file(
            file=b"raw file contents",
        )
        assert_matches_type(CatalogUploadFileResponse, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_upload_file(self, client: OctogenAPI) -> None:
        response = client.catalog.with_raw_response.upload_file(
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = response.parse()
        assert_matches_type(CatalogUploadFileResponse, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_upload_file(self, client: OctogenAPI) -> None:
        with client.catalog.with_streaming_response.upload_file(
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = response.parse()
            assert_matches_type(CatalogUploadFileResponse, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCatalog:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_agent_search(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.agent_search(
            text="text",
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_agent_search_with_all_params(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.agent_search(
            text="text",
            limit=0,
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_agent_search(self, async_client: AsyncOctogenAPI) -> None:
        response = await async_client.catalog.with_raw_response.agent_search(
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = await response.parse()
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_agent_search(self, async_client: AsyncOctogenAPI) -> None:
        async with async_client.catalog.with_streaming_response.agent_search(
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = await response.parse()
            assert_matches_type(SearchToolOutput, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_file(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.retrieve_file(
            "file_id",
        )
        assert_matches_type(object, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_file(self, async_client: AsyncOctogenAPI) -> None:
        response = await async_client.catalog.with_raw_response.retrieve_file(
            "file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = await response.parse()
        assert_matches_type(object, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_file(self, async_client: AsyncOctogenAPI) -> None:
        async with async_client.catalog.with_streaming_response.retrieve_file(
            "file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = await response.parse()
            assert_matches_type(object, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve_file(self, async_client: AsyncOctogenAPI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            await async_client.catalog.with_raw_response.retrieve_file(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_style_and_tags_search(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_style_and_tags_search_with_all_params(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
            compact_mode="compact",
            limit=0,
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_style_and_tags_search(self, async_client: AsyncOctogenAPI) -> None:
        response = await async_client.catalog.with_raw_response.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = await response.parse()
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_style_and_tags_search(self, async_client: AsyncOctogenAPI) -> None:
        async with async_client.catalog.with_streaming_response.style_and_tags_search(
            type="type",
            styles=["string"],
            tags=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = await response.parse()
            assert_matches_type(SearchToolOutput, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_text_search(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.text_search(
            text="text",
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_text_search_with_all_params(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.text_search(
            text="text",
            facets=[
                {
                    "name": "brand_name",
                    "values": ["string"],
                }
            ],
            limit=0,
            price_max=0,
            price_min=0,
        )
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_text_search(self, async_client: AsyncOctogenAPI) -> None:
        response = await async_client.catalog.with_raw_response.text_search(
            text="text",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = await response.parse()
        assert_matches_type(SearchToolOutput, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_text_search(self, async_client: AsyncOctogenAPI) -> None:
        async with async_client.catalog.with_streaming_response.text_search(
            text="text",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = await response.parse()
            assert_matches_type(SearchToolOutput, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_upload_file(self, async_client: AsyncOctogenAPI) -> None:
        catalog = await async_client.catalog.upload_file(
            file=b"raw file contents",
        )
        assert_matches_type(CatalogUploadFileResponse, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_upload_file(self, async_client: AsyncOctogenAPI) -> None:
        response = await async_client.catalog.with_raw_response.upload_file(
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        catalog = await response.parse()
        assert_matches_type(CatalogUploadFileResponse, catalog, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_upload_file(self, async_client: AsyncOctogenAPI) -> None:
        async with async_client.catalog.with_streaming_response.upload_file(
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            catalog = await response.parse()
            assert_matches_type(CatalogUploadFileResponse, catalog, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ContextEnum"]

ContextEnum: TypeAlias = Literal["https://schema.org/", "http://schema.org/", "https://schema.org", "http://schema.org"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ColorInfo", "Color"]


class Color(BaseModel):
    label: str
    """The color display name, or label.

    This may differ from standard color family names.
    """

    swatch_url: Optional[str] = None
    """A URL pointing to the color swatch image."""


class ColorInfo(BaseModel):
    color_families: Optional[List[str]] = None
    """Standard color families, such as 'Red', 'Green', 'Blue'. Maximum 5 values."""

    colors: Optional[List[Color]] = None
    """Color display names, which may differ from standard color family names.

    Maximum 75 values.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CatalogAgentSearchParams"]


class CatalogAgentSearchParams(TypedDict, total=False):
    text: Required[str]
    """Query text to be input to an LLM to generate a TextSearchQuery object"""

    limit: int
    """The maximum number of results to return from the search."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .brand import Brand as Brand
from .image import Image as Image
from .offer import Offer as Offer
from .offers import Offers as Offers
from .rating import Rating as Rating
from .review import Review as Review
from .audience import Audience as Audience
from .category import Category as Category
from .promotion import Promotion as Promotion
from .color_info import ColorInfo as ColorInfo
from .context_enum import ContextEnum as ContextEnum
from .organization import Organization as Organization
from .video_object import VideoObject as VideoObject
from .three_d_model import ThreeDModel as ThreeDModel
from .aggregate_offer import AggregateOffer as AggregateOffer
from .breadcrumb_list import BreadcrumbList as BreadcrumbList
from .fulfillment_info import FulfillmentInfo as FulfillmentInfo
from .product_enrichment import ProductEnrichment as ProductEnrichment
from .quantitative_value import QuantitativeValue as QuantitativeValue
from .search_tool_output import SearchToolOutput as SearchToolOutput
from .catalog_text_search_params import CatalogTextSearchParams as CatalogTextSearchParams
from .catalog_upload_file_params import CatalogUploadFileParams as CatalogUploadFileParams
from .catalog_agent_search_params import CatalogAgentSearchParams as CatalogAgentSearchParams
from .catalog_upload_file_response import CatalogUploadFileResponse as CatalogUploadFileResponse
from .catalog_style_and_tags_search_params import CatalogStyleAndTagsSearchParams as CatalogStyleAndTagsSearchParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from .brand import Brand
from .image import Image
from .offer import Offer
from .offers import Offers
from .rating import Rating
from .review import Review
from .._models import BaseModel
from .audience import Audience
from .category import Category
from .promotion import Promotion
from .color_info import ColorInfo
from .organization import Organization
from .video_object import VideoObject
from .three_d_model import ThreeDModel
from .aggregate_offer import AggregateOffer
from .breadcrumb_list import BreadcrumbList
from .fulfillment_info import FulfillmentInfo
from .product_enrichment import ProductEnrichment

__all__ = [
    "SearchToolOutput",
    "Product",
    "ProductAdditionalAttributes",
    "ProductBrand",
    "ProductHasVariant",
    "ProductHasVariantAdditionalAttributes",
    "ProductHasVariantBrand",
    "ProductHasVariantOffers",
    "ProductOffers",
]


class ProductAdditionalAttributes(BaseModel):
    numbers: Optional[List[float]] = None
    """Number attributes"""

    text: Optional[List[str]] = None
    """Text attributes"""


ProductBrand: TypeAlias = Union[Brand, Organization, None]


class ProductHasVariantAdditionalAttributes(BaseModel):
    numbers: Optional[List[float]] = None
    """Number attributes"""

    text: Optional[List[str]] = None
    """Text attributes"""


ProductHasVariantBrand: TypeAlias = Union[Brand, Organization, None]

ProductHasVariantOffers: TypeAlias = Union[Offer, AggregateOffer, Offers, None]


class ProductHasVariant(BaseModel):
    id: Optional[str] = None
    """Product identifier.

    Can be the identifier of a SKU as well. It should not be confused with any of
    the GTIN\\** numbers.
    """

    additional_attributes: Optional[Dict[str, Optional[ProductHasVariantAdditionalAttributes]]] = None
    """Extra product attributes."""

    audience: Optional[Audience] = None
    """Target group associated with the product."""

    brand: Optional[ProductHasVariantBrand] = None
    """The brand or organization associated with the product."""

    breadcrumb_list: Optional[BreadcrumbList] = FieldInfo(alias="breadcrumbList", default=None)
    """Breadcrumb list of the product."""

    catalog: Optional[str] = None
    """Catalog name, or ecommerce site name. Eg: Macys, sephora etc"""

    categories: Optional[List[Category]] = None
    """Product categories."""

    color_info: Optional[ColorInfo] = None
    """Color of the product."""

    description: Optional[str] = None
    """Product description."""

    dimensions: Optional[str] = None
    """The dimensions of the product."""

    enrichment: Optional[ProductEnrichment] = None
    """Enrichment information for the product.

    This is generated by the product information enrichment service.
    """

    extra_text: Optional[str] = None
    """Extra text field for additional information. Extracted from the web-page"""

    fit: Union[str, List[str], None] = None
    """The fit of the product."""

    fulfillment_info: Optional[List[FulfillmentInfo]] = None
    """Fulfillment information."""

    gtin: Optional[str] = None
    """Global Trade Item Number.

    a unique identifier used to recognize and track products globally in ecommerce,
    retail, and supply chains. It ensures that products are correctly identified
    across different marketplaces, point-of-sale (POS) systems, and logistics
    networks. Should not be confused with product or SKU id.
    """

    image: Optional[Image] = None
    """Primary product image."""

    images: Optional[List[Image]] = None
    """Product image."""

    language_code: Optional[str] = None
    """Language of the name and description."""

    materials: Optional[List[str]] = None
    """Material of the product."""

    name: Optional[str] = None
    """Full name or title of the product."""

    offers: Optional[ProductHasVariantOffers] = None
    """Offer or AggregateOffer for the product."""

    organization: Optional[Organization] = None
    """Schema.org model for Organization."""

    patterns: Optional[List[str]] = None
    """Pattern or graphic print of the product."""

    primary_product_id: Optional[str] = None
    """
    Used by Variant to refer to the productGroupID of the productGroup in which the
    product belongs
    """

    promotions: Optional[List[Promotion]] = None
    """Promotions applied to the product."""

    rating: Optional[Rating] = None
    """Product rating."""

    review: Optional[List[Review]] = None
    """List of reviews for the product."""

    sizes: Optional[List[str]] = None
    """Size of the product.

    A SKU has a single size. A product group can have a list of them.
    """

    tags: Optional[List[str]] = None
    """Custom tags associated with the product."""

    three_d_model: Optional[List[ThreeDModel]] = None
    """3D model of the product."""

    type: Optional[str] = None
    """The schema type."""

    url: Optional[str] = None
    """Canonical URL linking to the product detail page."""

    videos: Optional[List[VideoObject]] = None
    """Product video/videos."""


ProductOffers: TypeAlias = Union[Offer, AggregateOffer, Offers, None]


class Product(BaseModel):
    uuid: str

    id: Optional[str] = None
    """Product identifier.

    Can be the identifier of a SKU as well. It should not be confused with any of
    the GTIN\\** numbers.
    """

    additional_attributes: Optional[Dict[str, Optional[ProductAdditionalAttributes]]] = None
    """Extra product attributes."""

    audience: Optional[Audience] = None
    """Target group associated with the product."""

    brand: Optional[ProductBrand] = None
    """The brand or organization associated with the product."""

    brand_name: Optional[str] = None

    breadcrumb_list: Optional[BreadcrumbList] = FieldInfo(alias="breadcrumbList", default=None)
    """Breadcrumb list of the product."""

    catalog: Optional[str] = None
    """Catalog name, or ecommerce site name. Eg: Macys, sephora etc"""

    categories: Optional[List[Category]] = None
    """Product categories."""

    color_info: Optional[ColorInfo] = None
    """Color of the product."""

    current_price: Optional[float] = None

    description: Optional[str] = None
    """Product description."""

    dimensions: Optional[str] = None
    """The dimensions of the product."""

    enrichment: Optional[ProductEnrichment] = None
    """Enrichment information for the product.

    This is generated by the product information enrichment service.
    """

    extra_text: Optional[str] = None
    """Extra text field for additional information. Extracted from the web-page"""

    fit: Union[str, List[str], None] = None
    """The fit of the product."""

    fulfillment_info: Optional[List[FulfillmentInfo]] = None
    """Fulfillment information."""

    gtin: Optional[str] = None
    """Global Trade Item Number.

    a unique identifier used to recognize and track products globally in ecommerce,
    retail, and supply chains. It ensures that products are correctly identified
    across different marketplaces, point-of-sale (POS) systems, and logistics
    networks. Should not be confused with product or SKU id.
    """

    has_variant: Optional[List[ProductHasVariant]] = FieldInfo(alias="hasVariant", default=None)
    """Indicates a list of Products that are variants of this ProductGroup"""

    image: Optional[Image] = None
    """Primary product image."""

    images: Optional[List[Image]] = None
    """Product image."""

    language_code: Optional[str] = None
    """Language of the name and description."""

    materials: Optional[List[str]] = None
    """Material of the product."""

    name: Optional[str] = None
    """Full name or title of the product."""

    offers: Optional[ProductOffers] = None
    """Offer or AggregateOffer for the product."""

    organization: Optional[Organization] = None
    """Schema.org model for Organization."""

    original_price: Optional[float] = None

    patterns: Optional[List[str]] = None
    """Pattern or graphic print of the product."""

    primary_product_id: Optional[str] = None
    """
    Used by Variant to refer to the productGroupID of the productGroup in which the
    product belongs
    """

    product_group_id: Optional[str] = FieldInfo(alias="productGroupID", default=None)
    """ID of the product group."""

    promotions: Optional[List[Promotion]] = None
    """Promotions applied to the product."""

    rating: Optional[Rating] = None
    """Product rating."""

    review: Optional[List[Review]] = None
    """List of reviews for the product."""

    sizes: Optional[List[str]] = None
    """Size of the product.

    A SKU has a single size. A product group can have a list of them.
    """

    tags: Optional[List[str]] = None
    """Custom tags associated with the product."""

    three_d_model: Optional[List[ThreeDModel]] = None
    """3D model of the product."""

    type: Optional[str] = None
    """The schema type."""

    url: Optional[str] = None
    """Canonical URL linking to the product detail page."""

    varies_by: Optional[List[str]] = FieldInfo(alias="variesBy", default=None)
    """
    Indicates the property or properties by which the variants in a ProductGroup
    vary, e.g. their size, color, etc.
    """

    videos: Optional[List[VideoObject]] = None
    """Product video/videos."""


class SearchToolOutput(BaseModel):
    products: List[Product]
    """The list of product groups that match the input search query and filters."""

    total_found: int
    """The total number of products returned by the search."""

    error: Optional[str] = None
    """The error message if the search query is invalid or the search failed.

    The products field will be an empty list if the search fails and total_found
    will be 0.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .offer import Offer
from .._models import BaseModel
from .organization import Organization

__all__ = ["AggregateOffer"]


class AggregateOffer(BaseModel):
    availability: Optional[str] = None
    """
    The availability of the product — for example http://schema.org/InStock,
    http://schema.org/OutOfStock, http://schema.org/PreOrder, etc.
    """

    high_price: Optional[float] = FieldInfo(alias="highPrice", default=None)
    """Highest price of the offers."""

    item_condition: Optional[str] = FieldInfo(alias="itemCondition", default=None)
    """Condition of the items."""

    low_price: Optional[float] = FieldInfo(alias="lowPrice", default=None)
    """Lowest price of the offers."""

    offer_count: Optional[int] = FieldInfo(alias="offerCount", default=None)
    """Number of offers."""

    offers: Optional[List[Offer]] = None
    """List of individual offers."""

    price_currency: Optional[str] = FieldInfo(alias="priceCurrency", default=None)
    """Currency of the offers."""

    seller: Optional[Organization] = None
    """Schema.org model for Organization."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["CatalogUploadFileResponse"]


class CatalogUploadFileResponse(BaseModel):
    file_id: str

    file_url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["Audience"]


class Audience(BaseModel):
    age_groups: List[str]
    """The age groups of the audience.

    Suggested values: 'newborn', 'infant', 'toddler', 'kids', 'adult'. Maximum 5
    values.
    """

    genders: List[str]
    """The genders of the audience.

    Suggested values: 'male', 'female', 'unisex'. Maximum 5 values.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Organization", "ContactPoint", "ContactPointAddress"]


class ContactPointAddress(BaseModel):
    address_country: Optional[str] = FieldInfo(alias="addressCountry", default=None)
    """The country."""

    address_locality: Optional[str] = FieldInfo(alias="addressLocality", default=None)
    """The locality."""

    address_region: Optional[str] = FieldInfo(alias="addressRegion", default=None)
    """The region."""

    postal_code: Optional[str] = FieldInfo(alias="postalCode", default=None)
    """The postal code."""

    street_address: Optional[str] = FieldInfo(alias="streetAddress", default=None)
    """The street address."""

    type: Optional[str] = None


class ContactPoint(BaseModel):
    address: Optional[ContactPointAddress] = None
    """Schema.org model for PostalAddress."""

    contact_type: Optional[str] = FieldInfo(alias="contactType", default=None)
    """The type of contact point."""

    email: Optional[str] = None
    """The email address of the contact point."""

    telephone: Optional[str] = None
    """The telephone number of the contact point."""

    type: Optional[str] = None


class Organization(BaseModel):
    contact_point: Optional[ContactPoint] = FieldInfo(alias="contactPoint", default=None)
    """Schema.org model for ContactPoint."""

    context: Optional[str] = None

    logo: Optional[str] = None
    """The logo of the organization."""

    name: Optional[str] = None
    """The name of the organization."""

    same_as: Optional[List[str]] = FieldInfo(alias="sameAs", default=None)
    """The same as of the organization."""

    type: Optional[str] = None

    url: Optional[str] = None
    """The URL of the organization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ProductEnrichment"]


class ProductEnrichment(BaseModel):
    image_with_single_product: Optional[bool] = None
    """Whether the image only contains a single product."""

    styles: Optional[List[str]] = None
    """Styles for the product."""

    tags: Optional[List[str]] = None
    """Tags for the product."""

    type: Optional[str] = None
    """Type of the product."""

    type_synonyms: Optional[List[str]] = None
    """Synonyms for the type of the product."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["FulfillmentInfo"]


class FulfillmentInfo(BaseModel):
    place_ids: str
    """Store or region IDs for the fulfillment type.

    Must match the pattern '[a-zA-Z0-9_-]'.
    """

    type: str
    """
    Fulfillment type such as 'pickup-in-store', 'same-day-delivery', or a custom
    type.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .offer import Offer
from .._models import BaseModel

__all__ = ["Offers"]


class Offers(BaseModel):
    item_condition: Optional[str] = FieldInfo(alias="itemCondition", default=None)
    """The condition of the product (e.g., NewCondition, UsedCondition)."""

    offers: Optional[List[Offer]] = None
    """A list of individual offers for the product."""

    url: Optional[str] = None
    """The URL where the product can be purchased."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Image"]


class Image(BaseModel):
    url: str
    """Required.

    URI of the image. Must be a valid UTF-8 encoded URI with a maximum length of
    5000 characters.
    """

    gs_url: Optional[str] = None
    """Optional URL with the location of the image on google storage"""

    height: Optional[int] = None
    """Height of the image in pixels. Must be nonnegative."""

    size: Optional[int] = None
    """Size of the image in bytes. Must be nonnegative."""

    width: Optional[int] = None
    """Width of the image in pixels. Must be nonnegative."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .context_enum import ContextEnum

__all__ = ["VideoObject"]


class VideoObject(BaseModel):
    content_url: Optional[str] = FieldInfo(alias="contentUrl", default=None)
    """A URL pointing to the actual video content."""

    context: Optional[ContextEnum] = None

    description: Optional[str] = None
    """The description of the video."""

    duration: Optional[str] = None
    """The duration of the video in ISO 8601 format."""

    embed_url: Optional[str] = FieldInfo(alias="embedUrl", default=None)
    """A URL pointing to a player for the video."""

    interaction_count: Optional[int] = FieldInfo(alias="interactionCount", default=None)
    """The number of interactions for the video."""

    name: Optional[str] = None
    """The name of the video."""

    thumbnail_url: Optional[List[str]] = FieldInfo(alias="thumbnailUrl", default=None)
    """A URL pointing to the video thumbnail image."""

    type: Optional[str] = None

    upload_date: Optional[datetime] = FieldInfo(alias="uploadDate", default=None)
    """The date when the video was uploaded."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Mapping, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ..types import (
    catalog_text_search_params,
    catalog_upload_file_params,
    catalog_agent_search_params,
    catalog_style_and_tags_search_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.search_tool_output import SearchToolOutput
from ..types.catalog_upload_file_response import CatalogUploadFileResponse

__all__ = ["CatalogResource", "AsyncCatalogResource"]


class CatalogResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CatalogResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/octogen-systems/octogen-py-api#accessing-raw-response-data-eg-headers
        """
        return CatalogResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CatalogResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/octogen-systems/octogen-py-api#with_streaming_response
        """
        return CatalogResourceWithStreamingResponse(self)

    def agent_search(
        self,
        *,
        text: str,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchToolOutput:
        """
        Search for products using an LLM to generate a TextSearchQuery object that is
        used to search for products in the e-commerce catalog. The LLM will generate the
        fields of the TextSearchQuery object based on input query text.

        Args:
          text: Query text to be input to an LLM to generate a TextSearchQuery object

          limit: The maximum number of results to return from the search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/catalog/agent_search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "text": text,
                        "limit": limit,
                    },
                    catalog_agent_search_params.CatalogAgentSearchParams,
                ),
            ),
            cast_to=SearchToolOutput,
        )

    def retrieve_file(
        self,
        file_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Retrieve a file from Google Cloud Storage by file ID.

        Args: file_id: The unique identifier of the file

        Returns: StreamingResponse with the file content and appropriate content type

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not file_id:
            raise ValueError(f"Expected a non-empty value for `file_id` but received {file_id!r}")
        return self._get(
            f"/catalog/file/{file_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def style_and_tags_search(
        self,
        *,
        type: str,
        styles: List[str],
        tags: List[str],
        compact_mode: Optional[Literal["compact", "medium"]] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchToolOutput:
        """
        Search for products using the Octogen's search agent.

        Args: type: The type of product to search for styles: List of styles to search
        for tags: List of tags to search for limit: Maximum number of results to return.
        The default is 10.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/catalog/style_and_tags_search",
            body=maybe_transform(
                {
                    "styles": styles,
                    "tags": tags,
                },
                catalog_style_and_tags_search_params.CatalogStyleAndTagsSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "type": type,
                        "compact_mode": compact_mode,
                        "limit": limit,
                    },
                    catalog_style_and_tags_search_params.CatalogStyleAndTagsSearchParams,
                ),
            ),
            cast_to=SearchToolOutput,
        )

    def text_search(
        self,
        *,
        text: str,
        facets: Optional[Iterable[catalog_text_search_params.Facet]] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        price_max: Optional[float] | NotGiven = NOT_GIVEN,
        price_min: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchToolOutput:
        """Search for products in the Octogen e-commerce catalog.

        The search is performed
        using text embedding similarity on the input query text with the pre-computed
        product embeddings. The additional parameters are used filters to narrow down
        the search results. The number of results is determined by the limit parameter.

        Args:
          text: The text is converted to a vector embedding and used to search for products in
              the e-commerce catalog with pre-computed product embeddings.

          facets: The search results will be filtered by the specified facets.

          limit: The maximum number of results to return from the search. The default is 10.

          price_max: The products will be filtered to have a price less than or equal to the
              specified value.

          price_min: The products will be filtered to have a price greater than or equal to the
              specified value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/catalog/text_search",
            body=maybe_transform(
                {
                    "text": text,
                    "facets": facets,
                    "limit": limit,
                    "price_max": price_max,
                    "price_min": price_min,
                },
                catalog_text_search_params.CatalogTextSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SearchToolOutput,
        )

    def upload_file(
        self,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CatalogUploadFileResponse:
        """
        Upload an image file to Google Cloud Storage.

        Args: file: The file to upload

        Returns: FileUploadResponse with the file ID and URL

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/catalog/file_upload",
            body=maybe_transform(body, catalog_upload_file_params.CatalogUploadFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CatalogUploadFileResponse,
        )


class AsyncCatalogResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCatalogResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/octogen-systems/octogen-py-api#accessing-raw-response-data-eg-headers
        """
        return AsyncCatalogResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCatalogResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/octogen-systems/octogen-py-api#with_streaming_response
        """
        return AsyncCatalogResourceWithStreamingResponse(self)

    async def agent_search(
        self,
        *,
        text: str,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchToolOutput:
        """
        Search for products using an LLM to generate a TextSearchQuery object that is
        used to search for products in the e-commerce catalog. The LLM will generate the
        fields of the TextSearchQuery object based on input query text.

        Args:
          text: Query text to be input to an LLM to generate a TextSearchQuery object

          limit: The maximum number of results to return from the search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/catalog/agent_search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "text": text,
                        "limit": limit,
                    },
                    catalog_agent_search_params.CatalogAgentSearchParams,
                ),
            ),
            cast_to=SearchToolOutput,
        )

    async def retrieve_file(
        self,
        file_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Retrieve a file from Google Cloud Storage by file ID.

        Args: file_id: The unique identifier of the file

        Returns: StreamingResponse with the file content and appropriate content type

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not file_id:
            raise ValueError(f"Expected a non-empty value for `file_id` but received {file_id!r}")
        return await self._get(
            f"/catalog/file/{file_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def style_and_tags_search(
        self,
        *,
        type: str,
        styles: List[str],
        tags: List[str],
        compact_mode: Optional[Literal["compact", "medium"]] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchToolOutput:
        """
        Search for products using the Octogen's search agent.

        Args: type: The type of product to search for styles: List of styles to search
        for tags: List of tags to search for limit: Maximum number of results to return.
        The default is 10.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/catalog/style_and_tags_search",
            body=await async_maybe_transform(
                {
                    "styles": styles,
                    "tags": tags,
                },
                catalog_style_and_tags_search_params.CatalogStyleAndTagsSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "type": type,
                        "compact_mode": compact_mode,
                        "limit": limit,
                    },
                    catalog_style_and_tags_search_params.CatalogStyleAndTagsSearchParams,
                ),
            ),
            cast_to=SearchToolOutput,
        )

    async def text_search(
        self,
        *,
        text: str,
        facets: Optional[Iterable[catalog_text_search_params.Facet]] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        price_max: Optional[float] | NotGiven = NOT_GIVEN,
        price_min: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchToolOutput:
        """Search for products in the Octogen e-commerce catalog.

        The search is performed
        using text embedding similarity on the input query text with the pre-computed
        product embeddings. The additional parameters are used filters to narrow down
        the search results. The number of results is determined by the limit parameter.

        Args:
          text: The text is converted to a vector embedding and used to search for products in
              the e-commerce catalog with pre-computed product embeddings.

          facets: The search results will be filtered by the specified facets.

          limit: The maximum number of results to return from the search. The default is 10.

          price_max: The products will be filtered to have a price less than or equal to the
              specified value.

          price_min: The products will be filtered to have a price greater than or equal to the
              specified value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/catalog/text_search",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "facets": facets,
                    "limit": limit,
                    "price_max": price_max,
                    "price_min": price_min,
                },
                catalog_text_search_params.CatalogTextSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SearchToolOutput,
        )

    async def upload_file(
        self,
        *,
        file: FileTypes,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CatalogUploadFileResponse:
        """
        Upload an image file to Google Cloud Storage.

        Args: file: The file to upload

        Returns: FileUploadResponse with the file ID and URL

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({"file": file})
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/catalog/file_upload",
            body=await async_maybe_transform(body, catalog_upload_file_params.CatalogUploadFileParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CatalogUploadFileResponse,
        )


class CatalogResourceWithRawResponse:
    def __init__(self, catalog: CatalogResource) -> None:
        self._catalog = catalog

        self.agent_search = to_raw_response_wrapper(
            catalog.agent_search,
        )
        self.retrieve_file = to_raw_response_wrapper(
            catalog.retrieve_file,
        )
        self.style_and_tags_search = to_raw_response_wrapper(
            catalog.style_and_tags_search,
        )
        self.text_search = to_raw_response_wrapper(
            catalog.text_search,
        )
        self.upload_file = to_raw_response_wrapper(
            catalog.upload_file,
        )


class AsyncCatalogResourceWithRawResponse:
    def __init__(self, catalog: AsyncCatalogResource) -> None:
        self._catalog = catalog

        self.agent_search = async_to_raw_response_wrapper(
            catalog.agent_search,
        )
        self.retrieve_file = async_to_raw_response_wrapper(
            catalog.retrieve_file,
        )
        self.style_and_tags_search = async_to_raw_response_wrapper(
            catalog.style_and_tags_search,
        )
        self.text_search = async_to_raw_response_wrapper(
            catalog.text_search,
        )
        self.upload_file = async_to_raw_response_wrapper(
            catalog.upload_file,
        )


class CatalogResourceWithStreamingResponse:
    def __init__(self, catalog: CatalogResource) -> None:
        self._catalog = catalog

        self.agent_search = to_streamed_response_wrapper(
            catalog.agent_search,
        )
        self.retrieve_file = to_streamed_response_wrapper(
            catalog.retrieve_file,
        )
        self.style_and_tags_search = to_streamed_response_wrapper(
            catalog.style_and_tags_search,
        )
        self.text_search = to_streamed_response_wrapper(
            catalog.text_search,
        )
        self.upload_file = to_streamed_response_wrapper(
            catalog.upload_file,
        )


class AsyncCatalogResourceWithStreamingResponse:
    def __init__(self, catalog: AsyncCatalogResource) -> None:
        self._catalog = catalog

        self.agent_search = async_to_streamed_response_wrapper(
            catalog.agent_search,
        )
        self.retrieve_file = async_to_streamed_response_wrapper(
            catalog.retrieve_file,
        )
        self.style_and_tags_search = async_to_streamed_response_wrapper(
            catalog.style_and_tags_search,
        )
        self.text_search = async_to_streamed_response_wrapper(
            catalog.text_search,
        )
        self.upload_file = async_to_streamed_response_wrapper(
            catalog.upload_file,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import date

from pydantic import Field as FieldInfo

from .rating import Rating
from .._models import BaseModel

__all__ = ["Review", "Author"]


class Author(BaseModel):
    name: Optional[str] = None
    """The name of the person."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)


class Review(BaseModel):
    author: Optional[Author] = None
    """Schema.org Person definition."""

    date_published: Optional[date] = FieldInfo(alias="datePublished", default=None)
    """The date the review was published."""

    review_body: Optional[str] = FieldInfo(alias="reviewBody", default=None)
    """The body of the review."""

    review_rating: Optional[Rating] = FieldInfo(alias="reviewRating", default=None)
    """The rating given in this review."""

    type: Optional[str] = FieldInfo(alias="type_", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["QuantitativeValue"]


class QuantitativeValue(BaseModel):
    unit_code: Optional[str] = FieldInfo(alias="unitCode", default=None)
    """
    The unit of measurement given using the UN/CEFACT Common Code (3 characters) or
    a URL. Other codes than the UN/CEFACT Common Code may be used with a prefix
    followed by a colon..
    """

    value: Optional[float] = None
    """The value of the quantitative value."""
